use core::fmt;

use crate::query::ops::Operable;

use super::{qry_fn, QryFunc};

/// MetricsQL `limit_offset` query function
pub fn mql_limit_offset(
    limit: usize,
    offset: usize,
    qry_expr: impl Operable,
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result> {
    qry_fn!(limit_offset, "{limit},{offset},{qry_expr}")
}

/// MetricsQL `union` query function
pub fn mql_union(
    qry_expressions: &[impl Operable],
) -> QryFunc<impl Fn(&mut fmt::Formatter) -> fmt::Result + '_> {
    qry_fn!(
        union,
        "{queries}",
        queries = qry_expressions
            .iter()
            .map(ToString::to_string)
            .collect::<Vec<_>>()
            .join(",")
    )
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::query::Metric;

    #[test]
    fn trans_limit_offset() {
        let query = mql_limit_offset(1, 2, Metric::new("metric")).to_string();
        assert_eq!(query, "limit_offset(1,2,metric)");
    }

    #[test]
    fn trans_union() {
        let query = mql_union(&[Metric::new("metric"), Metric::new("metric2")]).to_string();
        assert_eq!(query, "union(metric,metric2)");
    }
}
